Object.defineProperty(exports, "__esModule", { value: true });
const uniq = require("lodash.uniq");
const isInteger = require("lodash.isinteger");
var ValidatesMethod;
(function (ValidatesMethod) {
    var Method;
    (function (Method) {
        function presence(errors, _, model, column) {
            const ids = model.where({ [column]: null }).pluck("id");
            if (ids.length !== 0)
                errors.push({ column, ids, message: "nullであるデータが存在します" });
        }
        Method.presence = presence;
        function presenceBelongsTo(errors, _, model, column) {
            const ids = model.where({ [column]: [null, undefined] }).pluck("id");
            if (ids.length !== 0)
                errors.push({ column: `${column}_id`, ids, message: "参照先がないデータが存在します" });
        }
        Method.presenceBelongsTo = presenceBelongsTo;
        function uniqueness(errors, _, model, column) {
            if (uniq(model.pluck(column)).length !== model.length) {
                const valuesMap = new Map();
                const idsMap = new Map();
                for (const record of model.toArray()) {
                    const value = record[column];
                    valuesMap.set(value, (valuesMap.get(value) || 0) + 1);
                    let ids = idsMap.get(value);
                    if (!ids) {
                        ids = [];
                        idsMap.set(value, ids);
                    }
                    ids.push(record.id);
                }
                for (const [value, count] of valuesMap.entries()) {
                    if (count > 1) {
                        const ids = idsMap.get(value);
                        errors.push({ column, ids, message: `値[${value}]がユニークではありません` });
                    }
                }
            }
        }
        Method.uniqueness = uniqueness;
        function length(errors, params, model, column) {
            const records = model.not({ [column]: null });
            if (params.is != null) {
                const ids = records.filterByColumn(column, (value) => value.length !== params.is).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `値が${params.is}ではありません` });
            }
            if (params.minimum != null) {
                const ids = records
                    .filterByColumn(column, (value) => value.length < params.minimum).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `値が${params.minimum}より小さいです` });
            }
            if (params.maximum != null) {
                const ids = records
                    .filterByColumn(column, (value) => value.length > params.maximum).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `値が${params.maximum}より大きいです` });
            }
        }
        Method.length = length;
        function format(errors, params, model, column) {
            const ids = model.not({ [column]: null })
                .filterByColumn(column, (value) => {
                if (params.with && !params.with.test(value))
                    return false;
                if (params.without && params.without.test(value))
                    return false;
                return true;
            })
                .pluck("id");
            if (ids.length) {
                errors.push({ column, ids, message: "フォーマットに反しています" });
            }
        }
        Method.format = format;
        function numericality(errors, params, model, column) {
            const records = model.not({ [column]: null });
            const nanIds = records
                .filterByColumn(column, (value) => typeof value !== "number" || isNaN(value)).pluck("id");
            if (nanIds.length) {
                errors.push({ column, ids: nanIds, message: "数値でないデータが存在します" });
            }
            if (params === true)
                return;
            if (params.only_integer) {
                const ids = records.filterByColumn(column, (value) => !isInteger(value)).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: "整数値でないデータが存在します" });
            }
            if (params.equal_to != null) {
                const ids = records.not({ [column]: params.equal_to }).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `[${params.equal_to}]と等しくないデータが存在します` });
            }
            if (params.greater_than != null) {
                const ids = records
                    .filterByColumn(column, (value) => value <= params.greater_than).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `[${params.greater_than}]以下のデータが存在します` });
            }
            if (params.greater_than_or_equal_to != null) {
                const ids = records.filterByColumn(column, (value) => value < params.greater_than_or_equal_to).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `[${params.greater_than_or_equal_to}]より小さいデータが存在します` });
            }
            if (params.less_than != null) {
                const ids = records
                    .filterByColumn(column, (value) => value >= params.less_than).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `[${params.less_than}]以上のデータが存在します` });
            }
            if (params.less_than_or_equal_to != null) {
                const ids = records
                    .filterByColumn(column, (value) => value > params.less_than_or_equal_to).pluck("id");
                if (ids.length)
                    errors.push({ column, ids, message: `[${params.less_than_or_equal_to}]より大きいデータが存在します` });
            }
        }
        Method.numericality = numericality;
        function inclusion(errors, params, model, column) {
            const ids = model.not({ [column]: params.in }).pluck("id");
            if (ids.length)
                errors.push({ column, ids, message: `[${params.in}]以外のデータが存在します` });
        }
        Method.inclusion = inclusion;
        function exclusion(errors, params, model, column) {
            const ids = model.where({ [column]: params.in }).pluck("id");
            if (ids.length)
                errors.push({ column, ids, message: `[${params.in}]にあたるデータが存在します` });
        }
        Method.exclusion = exclusion;
    })(Method = ValidatesMethod.Method || (ValidatesMethod.Method = {}));
})(ValidatesMethod = exports.ValidatesMethod || (exports.ValidatesMethod = {}));
//# sourceMappingURL=validates_method.js.map